@@ -5,7 +5,7 @@ use warnings;
 use Carp qw/croak/;
 use XSLoader;
 
-our $VERSION = '3.014_002'; # Don't forget to update the TestCompat set for testing against installed decoders!
+our $VERSION = '3.001'; # Don't forget to update the TestCompat set for testing against installed decoders!
 our $XS_VERSION = $VERSION; $VERSION= eval $VERSION;
 
 # not for public consumption, just for testing.
@@ -16,52 +16,9 @@ sub _test_compat {return(@$TestCompat, $VERSION)}
 # Make sure to keep these constants in sync with the C code in srl_encoder.c.
 # I know they could be exported from C using things like ExtUtils::Constant,
 # but that's too much of a hassle for just three numbers.
-use constant {
-    SRL_UNCOMPRESSED => 0,
-    SRL_SNAPPY       => 1,
-    SRL_ZLIB         => 2,
-};
-use constant #begin generated
-{
-  'SRL_F_ALIASED_DEDUPE_STRINGS' => 4096,
-  'SRL_F_CANONICAL_REFS' => 32768,
-  'SRL_F_COMPRESS_SNAPPY' => 64,
-  'SRL_F_COMPRESS_SNAPPY_INCREMENTAL' => 128,
-  'SRL_F_COMPRESS_ZLIB' => 256,
-  'SRL_F_CROAK_ON_BLESS' => 4,
-  'SRL_F_DEDUPE_STRINGS' => 2048,
-  'SRL_F_ENABLE_FREEZE_SUPPORT' => 16384,
-  'SRL_F_NOWARN_UNKNOWN_OVERLOAD' => 512,
-  'SRL_F_NO_BLESS_OBJECTS' => 8192,
-  'SRL_F_REUSE_ENCODER' => 2,
-  'SRL_F_SHARED_HASHKEYS' => 1,
-  'SRL_F_SORT_KEYS' => 1024,
-  'SRL_F_SORT_KEYS_PERL' => 65536,
-  'SRL_F_SORT_KEYS_PERL_REV' => 131072,
-  'SRL_F_STRINGIFY_UNKNOWN' => 16,
-  'SRL_F_UNDEF_UNKNOWN' => 8,
-  'SRL_F_WARN_UNKNOWN' => 32,
-  '_FLAG_NAME' => [
-                    'SHARED_HASHKEYS',
-                    'REUSE',
-                    'CROAK_ON_BLESS',
-                    'UNDEF_UNKNOWN',
-                    'STRINGIFY_UNKNOWN',
-                    'WARN_UNKNOWN',
-                    'COMPRESS_SNAPPY',
-                    'COMPRESS_SNAPPY_INCREMENTAL',
-                    'COMPRESS_ZLIB',
-                    'NOWARN_UNKNOWN_OVERLOAD',
-                    'SORT_KEYS',
-                    'DEDUPE_STRINGS',
-                    'ALIASED_DEDUPE_STRINGS',
-                    'NO_BLESS_OBJECTS',
-                    'ENABLE_FREEZE_SUPPORT',
-                    'CANONICAL_REFS',
-                    'SORT_KEYS_PERL',
-                    'SORT_KEYS_PERL_REV'
-                  ]
-}; #end generated
+use constant SRL_UNCOMPRESSED => 0;
+use constant SRL_SNAPPY       => 1;
+use constant SRL_ZLIB         => 2;
 
 use Exporter 'import';
 our @EXPORT_OK = qw(
@@ -80,29 +37,6 @@ sub CLONE_SKIP {1}
 
 XSLoader::load('Sereal::Encoder', $XS_VERSION);
 
-sub encode_to_file {
-    my ($self, $file, $struct, $append)= @_;
-    my $mode= $append ? ">>" : ">";
-    open my $fh, $mode, $file
-        or die "Failed to open '$file' for " . ($append ? "append" : "write") . ": $!";
-    print $fh $self->encode($struct)
-        or die "Failed to print to '$file': $!";
-    close $fh
-        or die "Failed to close '$file': $!";
-}
-
-my $flags= sub {
-    my ($int, $ary)= @_;
-    return map {
-        ($ary->[$_] and $int & (1 << $_)) ? $ary->[$_] : ()
-    } (0..$#$ary);
-};
-
-sub flag_names {
-    my ($self, $val)= @_;
-    return $flags->(defined $val ? $val : $self->flags, _FLAG_NAME);
-}
-
 1;
 
 __END__
@@ -116,13 +50,13 @@ Sereal::Encoder - Fast, compact, powerful binary serialization
 =head1 SYNOPSIS
 
   use Sereal::Encoder qw(encode_sereal sereal_encode_with_object);
-
+  
   my $encoder = Sereal::Encoder->new({...options...});
   my $out = $encoder->encode($structure);
-
+  
   # alternatively the functional interface:
   $out = sereal_encode_with_object($encoder, $structure);
-
+  
   # much slower functional interface with no persistent objects:
   $out = encode_sereal($structure, {... options ...});
 
@@ -178,7 +112,7 @@ are ignored. They are otherwise recognized for compatibility only.
 The size threshold (in bytes) of the uncompressed output below which
 compression is not even attempted even if enabled.
 Defaults to one kilobyte (1024 bytes). Set this to 0 and C<compress> to
-a non-C<SRL_UNCOMPRESSED> value to always attempt to compress.
+a non-zero value to always attempt to compress.
 Note that the document will not be compressed if the resulting size
 will be bigger than the original size (even if C<compress_threshold> is 0).
 
@@ -251,7 +185,7 @@ are documented below under L</"FREEZE/THAW CALLBACK MECHANISM">.
 Beware that using this functionality means a significant slowdown for
 object serialization. Even when serializing objects without a C<FREEZE>
 method, the additional method look up will cost a small amount of runtime.
-Yes, C<Sereal::Encoder> is so fast that this may make a difference.
+Yes, C<Sereal::Encoder> is so fast that is may make a difference.
 
 =head3 no_bless_objects
 
@@ -302,30 +236,6 @@ do so.
 Do note that the setting is somewhat approximate. Setting it to 10000 may break at
 somewhere between 9997 and 10003 nested structures depending on their types.
 
-=head3 canonical
-
-Enable all options which are related to producing canonical output, so that
-two strucutures with similar contents produce the same serialized form.
-
-See the caveats elsewhere in this document about producing canonical output.
-
-Currently sets the default for the following parameters: C<canonical_refs>
-and C<sort_keys>. If the option is explicitly set then this setting is ignored.
-More options may be added in the future.
-
-You are warned that use of this option may incur additional performance penalties
-in a future release by enabling other options than those listed here.
-
-=head3 canonical_refs
-
-Normally C<Sereal::Encoder> will ARRAYREF and HASHREF tags when the item contains
-less than 16 items, and and is not referenced more than once. This flag will
-override this optimization and use a standard REFN ARRAY style tag output. This
-is primarily useful for producing canonical output and for testing Sereal itself.
-
-See L</CANONICAL REPRESENTATION> for why you might want to use this, and
-for the various caveats involved.
-
 =head3 sort_keys
 
 Normally C<Sereal::Encoder> will output hashes in whatever order is convenient,
@@ -341,26 +251,14 @@ variables on use, and some of its rules are a little arcane (for instance utf8
 keys), and so two hashes that might appear to be the same might still produce
 different output as far as Sereal is concerned.
 
-As of 3.006_007 (prerelease candidate for 3.007) the sort order has been changed
-to the following: order by length of keys (in bytes) ascending, then by byte
-order of the raw underlying string, then by utf8ness, with non-utf8 first. This
-order was chosen because it is the most efficient to implement, both in terms
-of memory and time. This sort order is enabled when sort_keys is set to 1.
+The thusly allocated encoder object and its output buffer will be reused
+between invocations of C<encode()>, so hold on to it for an efficiency
+gain if you plan to serialize multiple similar data structures, but destroy
+it if you serialize a single very large data structure just once to free
+the memory.
 
-You may also produce output in Perl "cmp" order, by setting sort_keys to 2.
-And for backwards compatibility you may also produce output in reverse Perl
-"cmp" order by setting sort_keys to 3. Prior to 3.006_007 this was the
-only sort order possible, although it was not explicitly defined what it was.
-
-Note that comparatively speaking both of the "cmp" sort orders are slow and
-memory inefficient. Unless you have a really good reason stick to the default
-which is fast and as lean as possible.
-
-Unless you are concerned with "cross process canonical representation" then
-it doesn't matter what option you choose. 
-
-See L</CANONICAL REPRESENTATION> for why you might want to use this, and
-for the various caveats involved.
+See L</NON-CANONICAL> for why you might want to use this, and for the
+various caveats involved.
 
 =head3 no_shared_hashkeys
 
@@ -520,16 +418,16 @@ Here is a contrived example of a class implementing the C<FREEZE> / C<THAW> mech
 
   package
     File;
-
+  
   use Moo;
-
+  
   has 'path' => (is => 'ro');
   has 'fh' => (is => 'rw');
-
+  
   # open file handle if necessary and return it
   sub get_fh {
     my $self = shift;
-    # This could also be done with fancier Moo(se) syntax
+    # This could also with fancier Moo(se) syntax
     my $fh = $self->fh;
     if (not $fh) {
       open $fh, "<", $self->path or die $!;
@@ -537,7 +435,7 @@ Here is a contrived example of a class implementing the C<FREEZE> / C<THAW> mech
     }
     return $fh;
   }
-
+  
   sub FREEZE {
     my ($self, $serializer) = @_;
     # Could switch on $serializer here: JSON, CBOR, Sereal, ...
@@ -546,7 +444,7 @@ Here is a contrived example of a class implementing the C<FREEZE> / C<THAW> mech
     # to recreate.
     return $self->path;
   }
-
+  
   sub THAW {
     my ($class, $serializer, $data) = @_;
     # Turn back into object.
@@ -572,73 +470,29 @@ C<Sereal::Encoder> objects will become a reference to undef in the new
 thread. This might change in a future release to become a full clone
 of the encoder object.
 
-=head1 CANONICAL REPRESENTATION
+=head1 NON-CANONICAL 
 
 You might want to compare two data structures by comparing their serialized
 byte strings.  For that to work reliably the serialization must take extra
 steps to ensure that identical data structures are encoded into identical
 serialized byte strings (a so-called "canonical representation").
 
-Unfortunately in Perl there is no such thing as a "canonical representation".
-Most people are interested in "structural equivalence" but even that is less
-well defined than most people think. For instance in the following example:
+Currently the Sereal encoder I<does not> provide a mode that will reliably
+generate a canonical representation of a data structure. The reasons are many
+and sometimes subtle.
 
-    my $array1= [ 0, 0 ];
-    my $array2= do {
-        my $zero= 0;
-        sub{ \@_ }->($zero,$zero);
-    };
-
-the question of whether C<$array1> is structurally equivalent to C<$array2>
-is a subjective one. Sereal for instance would B<NOT> consider them
-equivalent but C<Test::Deep> would.  There are many examples of this in
-Perl. Simply stringifying a number technically changes the scalar. Storable
-would notice this, but Sereal generally would not.
-
-Despite this as of 3.002 the Sereal encoder supports a "canonical" option
-which will make a "best effort" attempt at producing a canonical
-representation of a data structure.  This mode is actually a combination of
-several other modes which may also be enabled independently, and as and when
-we add new options to the encoder that would assist in this regard then
-the C<canonical> will also enable them. These options may come with a
-performance penalty so care should be taken to read the Changes file and
-test the peformance implications when upgrading a system that uses this
-option.
-
-It is important to note that using canonical representation to determine
-if two data structures are different is subject to false-positives. If
-two Sereal encodings are identical you can generally assume that the
-two data structures are functionally equivalent from the point of view of
-normal Perl code (XS code might disagree). However if two Sereal
-encodings differ the data structures may actually be functionally
-equivalent.  In practice it seems the the false-positive rate is low,
-but your milage may vary.
-
-Some of the issues with producing a true canonical representation are
-outlined below:
+Sereal does support some use-cases however. In this section we attempt to outline
+the issues well enough for you to decide if it is suitable for your needs.
 
 =over 4
 
 =item Sereal doesn't order the hash keys by default.
 
-This can be enabled via the C<sort_keys>, which is itself enabled by
-C<canonical> option.
-
-=item Sereal output is sensitive to refcounts
-
-This can be somewhat mitigated by the use of C<canonical_refs>, see above.
+This can be enabled via C<sort_keys>, see above.
 
 =item There are multiple valid Sereal documents that you can produce for the same Perl data structure.
 
-Just L<sorting hash keys|/sort_keys> is not enough.  Some of the reasons
-are outlined below. These issues are especially relevant when considering
-language interoperability.
-
-=over 4
-
-=item PAD bytes
-
-A trivial example is PAD bytes which
+Just L<sorting hash keys|/sort_keys> is not enough. A trivial example is PAD bytes which
 mean nothing and are skipped. They mostly exist for encoder optimizations to
 prevent certain nasty backtracking situations from becoming O(n) at the cost of
 one byte of output. An explicit canonical mode would have to outlaw them (or
@@ -647,8 +501,6 @@ refcount/weakref handing in the encoder while at the same time causing some
 operations to go from O(1) to a full memcpy of everything after the point of
 where we backtracked to. Nasty.
 
-=item COPY tag
-
 Another example is COPY. The COPY tag indicates that the next element is an
 identical copy of a previous element (which is itself forbidden from including
 COPY's other than for class names). COPY is purely internal. The Perl/XS
@@ -656,8 +508,6 @@ implementation uses it to share hash keys and class names. One could use it for
 other strings (theoretically), but doesn't for time-efficiency reasons. We'd
 have to outlaw the use of this (significant) optimization of canonicalization.
 
-=item REF representation
-
 Sereal represents a reference to an array as a sequence of
 tags which, in its simplest form, reads I<REF, ARRAY $array_length TAG1 TAG2 ...>.
 The separation of "REF" and "ARRAY" is necessary to properly implement all of
@@ -668,10 +518,7 @@ into a special one byte ARRAYREF tag. This is a very significant optimization
 for common cases. This, however, does mean that most arrays up to 15 elements
 could be represented in two different, yet perfectly valid forms. ARRAYREF would
 have to be outlawed for a properly canonical form. The exact same logic
-applies to HASH vs. HASHREF. This behavior can be overridden by the
-C<canonical_refs> option, which disables use of HASHREF and ARRAYREF.
-
-=item Numeric representation
+applies to HASH vs. HASHREF.
 
 Similar to how Sereal can represent arrays and hashes in a full and a compact
 form. For small integers (between -16 and +15 inclusive), Sereal emits only
@@ -699,66 +546,7 @@ strings due to insignificant 'noise' in the floating point representation. Serea
 supports different floating point precisions and will generally choose the most
 compact that can represent your floating point number correctly.
 
-=back
-
-There's also a few cases where Sereal will produce different documents
-for values that you might think are the same thing, because if you
-e.g. compared them with C<eq> or C<==> in perl itself would think they
-were equivalent. However for the purposes of serialization they're not
-the same value.
-
-A good example of these cases is where L<Test::Deep> and Sereal's
-canonical mode differ. We have tests for some of these cases in
-F<t/030_canonical_vs_test_deep.t>. Here's the issues we've noticed so
-far:
-
-=over 4
-
-=item Sereal considers ASCII strings with the UTF-8 flag to be different from the same string without the UTF-8 flag
-
-Consider:
-
-    my $language_code = "en";
-
-v.s.:
-
-    my $language_code = "en";
-    utf8::upgrade($en);
-
-Sereal's canonical mode will encode these strings differently, as it
-should, since the UTF-8 flag will be passed along on interpolation.
-
-But this can be confusing if you're just getting some user-supplied
-ASCII strings that you may inadvertently toggle the UTF-8 flag on,
-e.g. because you're comparing an ASCII value in a database to a value
-submitted in a UTF-8 web form.
-
-=item Sereal will encode strings that look like numbers as strings, unless they've been used in numeric context
-
-I.e. these values will be encoded differently, respectively:
-
-    my $IV_x = "12345";
-    my $IV_y = "12345" + 0;
-    my $NV_x = "12.345";
-    my $NV_y = "12.345" + 0;
-
-But as noted above something like Test::Deep will consider these to be
-the same thing.
-
-=back
-
-We might produce certain aggressive flags to the canonical mode in the
-future to deal with this. For the cases noted above some combination
-of turning the UTF-8 flag on on all strings, or stripping it from
-strings that have it but are ASCII-only would "work", similarly we
-could scan strings to see if they match C<looks_like_number()> and if
-so numify them.
-
-This would produce output that either would be a lot bigger (having to
-encode all numbers as strings), or would be more expensive to generate
-(having to scan strings for numeric or non-ASCII context), and for
-some cases like the UTF-8 flag munging wouldn't be suitable for
-general use outside of canonicialization.
+These issues are especially relevant when considering language interoperability.
 
 =back
 
@@ -766,56 +554,11 @@ Often, people don't actually care about "canonical" in the strict sense
 required for real I<identity> checking. They just require a best-effort sort of
 thing for caching. But it's a slippery slope!
 
-In a nutshell, the C<canonical> option may be sufficient for an application
+In a nutshell, the C<sort_keys> option may be sufficient for an application
 which is simply serializing a cache key, and thus there's little harm in an
 occasional false-negative, but think carefully before applying Sereal in other
 use-cases.
 
-=head1 KNOWN ISSUES
-
-=over 4
-
-=item Strings Or Numbers
-
-Perl does not make a strong distinction between strings and numbers, and from
-an internal point of view it can be difficult to tell what the "right"
-representation is for a given variable.
-
-Sereal tries to not be lossy. So if it detects that the string value of a var,
-and the numeric value are different it will generally round trip the *string*
-value. This means that "special" strings often used in Perl function returns,
-like "0 but true", and "0e0", will round trip in a way that their normal Perl
-semantics are preserved. However this also means that "non canonical" values,
-like " 100 ", which will numify as 100 without warnings, will round trip as
-their string values.
-
-Perl also has some operators, the binary operators, ^, | and &, which do different
-things depending on whether their arguments had been used in numeric context as
-the following examples show:
-
-    perl -le'my $x="1"; $i=int($x); print unpack "H*", $x ^ "1"'
-    30
-
-    perl -le'my $x="1"; print unpack "H*", $x ^ "1"'
-    00
-
-    perl -le'my $x=" 1 "; $i=int($x); print unpack "H*", $x ^ "1"'
-    30
-
-    perl -le'my $x=" 1 "; print unpack "H*", $x ^ "1"'
-    113120
-
-Sereal currently cannot round trip this property properly.
-
-An extreme case of this problem is that of "dualvars", which can be created using
-the Scalar::Util::dualvar() function. This function allows one to create variables
-which have string and integer values which are completely unrelated to each other.
-Sereal currently will choose the *string* value when it detects these items.
-
-It is possible that a future release of the protocol will fix these issues.
-
-=back
-
 =head1 BUGS, CONTACT AND SUPPORT
 
 For reporting bugs, please use the github bug tracker at
